==> migrate_orphans-1.0.3.tar.gz: migrate_orphans/src/Commands/MigrateOrphansCommands.php <==
<?php

namespace Drupal\migrate_orphans\Commands;

use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\Plugin\MigrationPluginManager;
use Drupal\migrate_tools\Commands\MigrateToolsCommands;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://cgit.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://cgit.drupalcode.org/devel/tree/drush.services.yml
 */
class MigrateOrphansCommands extends MigrateToolsCommands {

  use StringTranslationTrait;

  /**
   * Metatag logging channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  private $connection;

  /**
   * MigrateToolsCommands constructor.
   *
   * @param \Drupal\migrate\Plugin\MigrationPluginManager $migrationPluginManager
   *   Migration Plugin Manager service.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   Date formatter service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager service.
   * @param \Drupal\Core\KeyValueStore\KeyValueFactoryInterface $keyValue
   *   Key-value store service.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $channelFactory
   *   The LoggerChannelFactoryInterface object.
   */
  public function __construct(
    MigrationPluginManager $migrationPluginManager,
    DateFormatter $dateFormatter,
    EntityTypeManagerInterface $entityTypeManager,
    KeyValueFactoryInterface $keyValue,
    LoggerChannelFactoryInterface $loggerFactory,
    Connection $connection
  ) {
    parent::__construct($migrationPluginManager, $dateFormatter, $entityTypeManager, $keyValue);
    $this->logger = $loggerFactory->get('migrate_orphans');
    $this->connection = $connection;
  }

  /**
   * Remove imported records that has been removed in origin source
   *
   * @param $migration_name
   *   Name of the migration to check
   * @param array $options An associative array of options whose values come from cli, aliases, config, etc.
   *
   * @option dry-run
   *   Do not delete, just report the records
   * @usage drush migrate-orphans-purge migration
   *   Removes orphans from migration
   *
   * @command migrate:orphans-purge
   * @aliases morp,migrate-orphans-purge
   *
   * @throws \Exception
   *  If there are not enough parameters to the command.
   */
  public function orphansPurge($migration_name, array $options = ['dry-run' => NULL, 'delete' => TRUE]) {
    $migrations = $this->migrationsList($migration_name, $options);

    if (empty($migrations)) {
      $this->logger->error(($this->t('No migrations found.')));
    }
    foreach ($migrations as $migration_list) {
      /** @var \Drupal\migrate\Plugin\Migration $migration */
      foreach ($migration_list as $migration) {
        if ($migration->getStatus() != MigrationInterface::STATUS_IDLE) {
          throw new \Exception(dt('Migration not idle, waiting to finish. Status is: ' . $migration->getStatus()));
        }

        $rows = $this->connection->select('migrate_map_' . strtolower($migration->getPluginId()), 'm')
          ->fields('m', ['sourceid1', 'destid1'])
          ->isNotNull('destid1')
          ->execute()
          ->fetchAllKeyed(0, 1);

        $count = count($rows);
        $orphans = $rows;

        $dry_run = $options['dry-run'];

        $this->logger->notice("Total migrated: " . $count);

        /** @var \Drupal\migrate_plus\Plugin\migrate\source\Url $source */
        $source = $migration->getSourcePlugin();

        if (get_class($source) == 'Drupal\migrate_plus\Plugin\migrate\source\Url' ||
          is_subclass_of(get_class($source), 'Drupal\migrate_plus\Plugin\migrate\source\Url')) {
          // Init first source:
          /** @var \Drupal\migrate_plus\Annotation\DataParser */
          $dataparser = $source->getDataParserPlugin();
          $dataparser->rewind();

          while ($dataparser->valid()) {
            $key = $dataparser->key();
            $id = reset($key);
            unset($orphans[$id]);
            $dataparser->next();
          }
        }
        else {
          $this->logger->error('Migrate class ' . get_class($source) . ' not compatible.', 'error');
          return;
        }

        /** @var \Drupal\migrate\Plugin\migrate\destination\Entity $destination */
        $destination = $migration->getDestinationPlugin();

        $this->logger->notice('Total orphans: ' . count($orphans));

        if (!empty($orphans)) {
          $deleted = $disabled = FALSE;
          if (!$dry_run && $options['delete']) {
            if (!empty($destination->getPluginDefinition()['id'])) {
              $provider = str_replace('entity:', '', $destination->getPluginDefinition()['id']);
              $controller = $this->entityTypeManager->getStorage($provider);
              if (count($orphans) > 1000) {
                for ($i = 0; $i <= count($orphans); $i += 1000) {
                  $end = $i + 1000 > count($orphans) ? count($orphans) : $i + 1000;
                  $to_delete = array_slice($orphans, $i, $end);
                  $entities = $controller->loadMultiple($to_delete);
                  $controller->delete($entities);
                  $this->logger->notice('Deleted entites from ' . $i . ' to ' . $end);
                }
              }
              else {
                $entities = $controller->loadMultiple($orphans);
                $controller->delete($entities);
              }
              $deleted = TRUE;
            }
          }
          else {
            if (!$dry_run && !$options['delete']) {
              if (!empty($destination->getPluginDefinition()['id'])) {
                $provider = str_replace('entity:', '', $destination->getPluginDefinition()['id']);
                $controller = $this->entityTypeManager->getStorage($provider);
                $entities = $controller->loadMultiple($orphans);

                // Unpublish orphans
                foreach ($entities as $node) {
                  $node->status = 0;
                  $node->save();
                }
                $disabled = TRUE;
              }
            }
          }

          if ($deleted || $disabled) {
            foreach ($orphans as $current_id => $orphan) {
              $source_ids = $migration->getSourcePlugin()->getIds();
              $values = [];
              foreach (array_keys($source_ids) as $source_id) {
                $values[$source_id] = $current_id;
              }
              $migration->getIdMap()->delete($values);

            }
            $this->logger->success('All orphan elements have been removed');
          }

        }
        else {
          $this->logger->success('There are no more orphan elements');
        }
      }
    }
  }

  /**
   * Disable imported records that has been removed in origin source
   *
   * @param $migration_name
   *   Name of the migration to check
   * @param array $options An associative array of options whose values come from cli, aliases, config, etc.
   *
   * @option dry-run
   *   Do not disable, just report the records
   * @usage drush migrate-orphans-disbale migration
   *   Disables orphans from migration
   *
   * @command migrate:orphans-disable
   * @aliases mord,migrate-orphans-disable
   *
   * @throws \Exception
   *  If there are not enough parameters to the command.
   */
  public function orphansDisable($migration_name, array $options = ['dry-run' => NULL]) {
    $this->orphansPurge($migration_name, [$options['dry-run'], FALSE]);
  }

}

==> migrate_orphans-1.0.5.tar.gz: migrate_orphans/src/Commands/MigrateOrphansCommands.php <==
<?php

namespace Drupal\migrate_orphans\Commands;

use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\Plugin\MigrationPluginManager;
use Drupal\migrate_tools\Commands\MigrateToolsCommands;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://cgit.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://cgit.drupalcode.org/devel/tree/drush.services.yml
 */
class MigrateOrphansCommands extends MigrateToolsCommands {

  use StringTranslationTrait;

  /**
   * Metatag logging channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  private $connection;

  /**
   * MigrateToolsCommands constructor.
   *
   * @param \Drupal\migrate\Plugin\MigrationPluginManager $migrationPluginManager
   *   Migration Plugin Manager service.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   Date formatter service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager service.
   * @param \Drupal\Core\KeyValueStore\KeyValueFactoryInterface $keyValue
   *   Key-value store service.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $channelFactory
   *   The LoggerChannelFactoryInterface object.
   */
  public function __construct(
    MigrationPluginManager $migrationPluginManager,
    DateFormatter $dateFormatter,
    EntityTypeManagerInterface $entityTypeManager,
    KeyValueFactoryInterface $keyValue,
    LoggerChannelFactoryInterface $loggerFactory,
    Connection $connection
  ) {
    parent::__construct($migrationPluginManager, $dateFormatter, $entityTypeManager, $keyValue);
    $this->logger = $loggerFactory->get('migrate_orphans');
    $this->connection = $connection;
  }

  /**
   * Remove imported records that has been removed in origin source
   *
   * @param $migration_name
   *   Name of the migration to check
   * @param array $options An associative array of options whose values come from cli, aliases, config, etc.
   *
   * @option dry-run
   *   Do not delete, just report the records
   * @option group
   *   A comma-separated list of migration groups to list
   * @usage drush migrate-orphans-purge migration
   *   Removes orphans from migration
   *
   * @command migrate:orphans-purge
   * @aliases morp,migrate-orphans-purge
   *
   * @throws \Exception
   *  If there are not enough parameters to the command.
   */
  public function orphansPurge($migration_name, array $options = ['dry-run' => NULL, 'delete' => TRUE, 'group' => NULL]) {
    $migrations = $this->migrationsList($migration_name, $options);

    if (empty($migrations)) {
      $this->logger->error(($this->t('No migrations found.')));
    }

    foreach ($migrations as $migration_list) {
      /** @var \Drupal\migrate\Plugin\Migration $migration */
      foreach ($migration_list as $migration) {
        if ($migration->getStatus() != MigrationInterface::STATUS_IDLE) {
          throw new \Exception(dt('Migration not idle, waiting to finish. Status is: ' . $migration->getStatus()));
        }

        $rows = $this->connection->select('migrate_map_' . strtolower($migration->getPluginId()), 'm')
          ->fields('m', ['sourceid1', 'destid1'])
          ->isNotNull('destid1')
          ->execute()
          ->fetchAllKeyed(0, 1);

        $count = count($rows);
        $orphans = $rows;

        $dry_run = $options['dry-run'];

        $this->logger->notice("Total migrated: " . $count);

        /** @var \Drupal\migrate_plus\Plugin\migrate\source\Url $source */
        $source = $migration->getSourcePlugin();

        if (get_class($source) == 'Drupal\migrate_plus\Plugin\migrate\source\Url' ||
          is_subclass_of(get_class($source), 'Drupal\migrate_plus\Plugin\migrate\source\Url')) {
          // Init first source:
          /** @var \Drupal\migrate_plus\Annotation\DataParser */
          $dataparser = $source->getDataParserPlugin();
          $dataparser->rewind();

          while ($dataparser->valid()) {
            $key = $dataparser->key();
            $id = reset($key);
            unset($orphans[$id]);
            $dataparser->next();
          }
        }
        else {
          $this->logger->error('Migrate class ' . get_class($source) . ' not compatible.', 'error');
          return;
        }

        /** @var \Drupal\migrate\Plugin\migrate\destination\Entity $destination */
        $destination = $migration->getDestinationPlugin();

        $this->logger->notice('Total orphans: ' . count($orphans));

        if (!empty($orphans)) {
          $deleted = $disabled = FALSE;
          if (!$dry_run && $options['delete']) {
            if (!empty($destination->getPluginDefinition()['id'])) {
              $provider = str_replace('entity:', '', $destination->getPluginDefinition()['id']);
              $controller = $this->entityTypeManager->getStorage($provider);
              if (count($orphans) > 1000) {
                for ($i = 0; $i <= count($orphans); $i += 1000) {
                  $end = $i + 1000 > count($orphans) ? count($orphans) : $i + 1000;
                  $to_delete = array_slice($orphans, $i, $end);
                  $entities = $controller->loadMultiple($to_delete);
                  $controller->delete($entities);
                  $this->logger->notice('Deleted entites from ' . $i . ' to ' . $end);
                }
              }
              else {
                $entities = $controller->loadMultiple($orphans);
                $controller->delete($entities);
              }
              $deleted = TRUE;
            }
          }
          else {
            if (!$dry_run && !$options['delete']) {
              if (!empty($destination->getPluginDefinition()['id'])) {
                $provider = str_replace('entity:', '', $destination->getPluginDefinition()['id']);
                $controller = $this->entityTypeManager->getStorage($provider);
                $entities = $controller->loadMultiple($orphans);

                // Unpublish orphans
                foreach ($entities as $node) {
                  $node->status = 0;
                  $node->save();
                }
                $disabled = TRUE;
              }
            }
          }

          if ($deleted || $disabled) {
            foreach ($orphans as $current_id => $orphan) {
              $source_ids = $migration->getSourcePlugin()->getIds();
              $values = [];
              foreach (array_keys($source_ids) as $source_id) {
                $values[$source_id] = $current_id;
              }
              $migration->getIdMap()->delete($values);

            }
            $this->logger->success('All orphan elements have been removed');
          }

        }
        else {
          $this->logger->success('There are no more orphan elements');
        }
      }
    }
  }

  /**
   * Disable imported records that has been removed in origin source
   *
   * @param $migration_name
   *   Name of the migration to check
   * @param array $options An associative array of options whose values come from cli, aliases, config, etc.
   *
   * @option dry-run
   *   Do not disable, just report the records
   * @option group
   *   A comma-separated list of migration groups to list
   * @usage drush migrate-orphans-disbale migration
   *   Disables orphans from migration
   *
   * @command migrate:orphans-disable
   * @aliases mord,migrate-orphans-disable
   *
   * @throws \Exception
   *  If there are not enough parameters to the command.
   */
  public function orphansDisable($migration_name, array $options = ['dry-run' => NULL, 'group' => NULL]) {
    $this->orphansPurge($migration_name, [$options['dry-run'], FALSE]);
  }

}
